Imports System.Data
Imports System.Data.OleDb
Public Class frmAuthors
  Dim BooksConnection As OleDbConnection
  Dim AuthorsCommand As OleDbCommand
  Dim AuthorsAdapter As OleDbDataAdapter
  Dim AuthorsTable As DataTable
  Dim AuthorsManager As CurrencyManager

  Private Sub frmAuthors_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
    'close the connection
    BooksConnection.Close()
    'dispose of the objects
    BooksConnection.Dispose()
    AuthorsCommand.Dispose()
    AuthorsAdapter.Dispose()
    AuthorsTable.Dispose()
  End Sub

  Private Sub frmAuthors_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
    'connect to books database
    BooksConnection = New OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0; Data Source = c:\VB2005DB\Working\books.mdb")
    BooksConnection.Open()
    'establish command object
    AuthorsCommand = New OleDbCommand("Select * from Authors ORDER BY Author", BooksConnection)
    'establish data adapter/data table
    AuthorsAdapter = New OleDbDataAdapter()
    AuthorsAdapter.SelectCommand = AuthorsCommand
    AuthorsTable = New DataTable()
    AuthorsAdapter.Fill(AuthorsTable)
    'bind controls to data table
    txtAuthorID.DataBindings.Add("Text", AuthorsTable, "Au_ID")
    txtAuthorName.DataBindings.Add("Text", AuthorsTable, "Author")
    txtYearBorn.DataBindings.Add("Text", AuthorsTable, "Year_Born")
    'establish currency manager
    AuthorsManager = DirectCast(Me.BindingContext(AuthorsTable), CurrencyManager)
    Me.Show()
    Call SetState("View")
  End Sub
  Private Sub btnPrevious_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrevious.Click
    If AuthorsManager.Position = 0 Then Console.Beep()
    AuthorsManager.Position -= 1
  End Sub

  Private Sub btnNext_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnNext.Click
    If AuthorsManager.Position = AuthorsManager.Count - 1 Then Console.Beep()
    AuthorsManager.Position += 1
  End Sub

  Private Sub btnSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSave.Click
    If Not (ValidateData()) Then Exit Sub
    MessageBox.Show("Record saved.", "Save", MessageBoxButtons.OK, MessageBoxIcon.Information)
    Call SetState("View")
  End Sub

  Private Sub btnDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDelete.Click
    Dim Response As Windows.Forms.DialogResult
    Response = MessageBox.Show("Are you sure you want to delete this record?", "Delete", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2)
    If Response = Windows.Forms.DialogResult.No Then
      Exit Sub
    End If
    End Sub
  Private Sub SetState(ByVal AppState As String)
    Select Case AppState
      Case "View"
        txtAuthorID.BackColor = Color.White
        txtAuthorID.ForeColor = Color.Black
        txtAuthorName.ReadOnly = True
        txtYearBorn.ReadOnly = True
        btnPrevious.Enabled = True
        btnNext.Enabled = True
        btnAddNew.Enabled = True
        btnSave.Enabled = False
        btnCancel.Enabled = False
        btnEdit.Enabled = True
        btnDelete.Enabled = True
        btnDone.Enabled = True
        txtAuthorName.Focus()
      Case "Add", "Edit"
        txtAuthorID.BackColor = Color.Red
        txtAuthorID.ForeColor = Color.White
        txtAuthorName.ReadOnly = False
        txtYearBorn.ReadOnly = False
        btnPrevious.Enabled = False
        btnNext.Enabled = False
        btnAddNew.Enabled = False
        btnSave.Enabled = True
        btnCancel.Enabled = True
        btnEdit.Enabled = False
        btnDelete.Enabled = False
        btnDone.Enabled = False
        txtAuthorName.Focus()
    End Select
  End Sub

  Private Sub btnAddNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAddNew.Click
    Call SetState("Add")
  End Sub

  Private Sub btnEdit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnEdit.Click
    Call SetState("Edit")
  End Sub

  Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
    Call SetState("View")
  End Sub

  Private Sub txtYearBorn_KeyPress(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles txtYearBorn.KeyPress
    Select Case e.KeyChar
      Case CChar("0") To CChar("9"), ControlChars.Back
        'Acceptable keystrokes
        e.Handled = False
      Case ControlChars.Cr
        txtAuthorName.Focus()
      Case Else
        e.Handled = True
        Console.Beep()
    End Select
  End Sub

  Private Function ValidateData() As Boolean
    Dim Message As String = ""
    Dim InputYear As Integer, CurrentYear As Integer
    Dim AllOK As Boolean = True
    'Check for name
    If txtAuthorName.Text.Trim = "" Then
      Message = "You must enter an Author Name." + ControlChars.CrLf
      txtAuthorName.Focus()
      AllOK = False
    End If
    'Check length and range on Year Born
    If txtYearBorn.Text.Trim <> "" Then
      InputYear = CInt(Val(txtYearBorn.Text))
      CurrentYear = CInt(Val(Format(Now, "yyyy")))
      If InputYear > CurrentYear Or InputYear < CurrentYear - 150 Then
        Message += "Year born must be between" + Str(CurrentYear - 150) + " and" + Str(CurrentYear)
        txtYearBorn.Focus()
        AllOK = False
      End If
    End If
    If Not (AllOK) Then
      MessageBox.Show(Message, "Validation Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
    End If
    Return (AllOK)
  End Function

  Private Sub txtAuthorName_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles txtAuthorName.KeyPress
    If e.KeyChar = ControlChars.Cr Then
      txtYearBorn.Focus()
    End If
  End Sub
End Class
